# -*- coding: utf-8 -*-
# Copyright 2007-2016 The HyperSpy developers
#
# This file is part of  HyperSpy.
#
#  HyperSpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
#  HyperSpy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.

import math

import numpy as np

from hyperspy.component import Component


class PowerLaw(Component):

    """Power law component

    f(x) = A*(x-x0)^-r

    +------------+-----------+
    | Parameter  | Attribute |
    +------------+-----------+
    +------------+-----------+
    |     A      |     A     |
    +------------+-----------+
    |     r      |     r     |
    +------------+-----------+
    |    x0      |  origin   |
    +------------+-----------+

    The left_cutoff parameter can be used to set a lower threshold from which
    the component will return 0.


    """

    def __init__(self, A=10e5, r=3., origin=0.):
        Component.__init__(self, ('A', 'r', 'origin'))
        self.A.value = A
        self.r.value = r
        self.origin.value = origin
        self.origin.free = False
        self.left_cutoff = 0.

        # Boundaries
        self.A.bmin = 0.
        self.A.bmax = None
        self.r.bmin = 1.
        self.r.bmax = 5.

        self.isbackground = True
        self.convolved = False

    def function(self, x):
        return np.where(x > self.left_cutoff, self.A.value *
                        (x - self.origin.value) ** (-self.r.value), 0)

    def grad_A(self, x):
        return self.function(x) / self.A.value

    def grad_r(self, x):
        return np.where(x > self.left_cutoff, -self.A.value *
                        np.log(x - self.origin.value) *
                        (x - self.origin.value) ** (-self.r.value), 0)

    def grad_origin(self, x):
        return np.where(x > self.left_cutoff, self.r.value *
                        (x - self.origin.value) ** (-self.r.value - 1) *
                        self.A.value, 0)

    def estimate_parameters(self, signal, x1, x2, only_current=False):
        """Estimate the parameters by the two area method

        Parameters
        ----------
        signal : Signal instance
        x1 : float
            Defines the left limit of the spectral range to use for the
            estimation.
        x2 : float
            Defines the right limit of the spectral range to use for the
            estimation.

        only_current : bool
            If False estimates the parameters for the full dataset.

        Returns
        -------
        bool

        """
        super(PowerLaw, self)._estimate_parameters(signal)
        axis = signal.axes_manager.signal_axes[0]
        i1, i2 = axis.value_range_to_indices(x1, x2)
        if not (i2 + i1) % 2 == 0:
            i2 -= 1
        if i2 == i1:
            i2 += 2
        i3 = (i2 + i1) // 2
        x1 = axis.index2value(i1)
        x2 = axis.index2value(i2)
        x3 = axis.index2value(i3)
        if only_current is True:
            s = signal.get_current_signal()
        else:
            s = signal
        I1 = s.isig[i1:i3].integrate1D(2j).data.astype("float")
        I2 = s.isig[i3:i2].integrate1D(2j).data.astype("float")
        try:
            r = 2 * np.log(I1 / I2) / math.log(x2 / x1)
            k = 1 - r
            A = k * I2 / (x2 ** k - x3 ** k)
            r = np.nan_to_num(r)
            A = np.nan_to_num(A)
        except:
            return False
        if only_current is True:
            self.r.value = r
            self.A.value = A
            return True
        else:
            if self.A.map is None:
                self._create_arrays()
            self.A.map['values'][:] = A
            self.A.map['is_set'][:] = True
            self.r.map['values'][:] = r
            self.r.map['is_set'][:] = True
            self.fetch_stored_values()
            return True





import numpy as np

from hyperspy.misc.math_tools import get_linear_interpolation
from hyperspy.misc.elements import elements


class GOSBase(object):

    def read_elements(self):
        element = self.element
        subshell = self.subshell
        # Convert to the "GATAN" nomenclature
        if (element in elements) is not True:
            raise ValueError("The given element " + element +
                             " is not in the database.")
        elif subshell not in elements[element]['Atomic_properties']['Binding_energies']:
            raise ValueError(
                "The given subshell " + subshell +
                " is not in the database.\n" +
                "The available subshells are:\n" +
                str(list(elements[element]['Atomic_properties']['subshells'].keys())))

        self.onset_energy = \
            elements[
                element][
                'Atomic_properties'][
                'Binding_energies'][
                subshell][
                'onset_energy (eV)']
        self.subshell_factor = \
            elements[
                element][
                'Atomic_properties'][
                'Binding_energies'][
                subshell][
                'factor']
        self.Z = elements[element]['General_properties']['Z']
        self.element_dict = elements[element]

    def get_parametrized_qaxis(self, k1, k2, n):
        return k1 * (np.exp(np.arange(n) * k2) - 1) * 1e10

    def get_parametrized_energy_axis(self, k1, k2, n):
        return k1 * (np.exp(np.arange(n) * k2 / k1) - 1)

    def get_qaxis_and_gos(self, ienergy, qmin, qmax):
        qgosi = self.gos_array[ienergy, :]
        if qmax > self.qaxis[-1]:
            # Linear extrapolation
            g1, g2 = qgosi[-2:]
            q1, q2 = self.qaxis[-2:]
            gosqmax = get_linear_interpolation((q1, g1), (q2, g2), qmax)
            qaxis = np.hstack((self.qaxis, qmax))
            qgosi = np.hstack((qgosi, gosqmax))
        else:
            index = self.qaxis.searchsorted(qmax)
            g1, g2 = qgosi[index - 1:index + 1]
            q1, q2 = self.qaxis[index - 1: index + 1]
            gosqmax = get_linear_interpolation((q1, g1), (q2, g2), qmax)
            qaxis = np.hstack((self.qaxis[:index], qmax))
            qgosi = np.hstack((qgosi[:index, ], gosqmax))

        if qmin > 0:
            index = self.qaxis.searchsorted(qmin)
            g1, g2 = qgosi[index - 1:index + 1]
            q1, q2 = qaxis[index - 1:index + 1]
            gosqmin = get_linear_interpolation((q1, g1), (q2, g2), qmin)
            qaxis = np.hstack((qmin, qaxis[index:]))
            qgosi = np.hstack((gosqmin, qgosi[index:],))
        return qaxis, qgosi.clip(0)



import nose.tools as nt
import numpy as np

from hyperspy.misc.utils import DictionaryTreeBrowser
from hyperspy.signal import BaseSignal


class TestDictionaryBrowser:

    def setUp(self):
        tree = DictionaryTreeBrowser(
            {
                "Node1": {"leaf11": 11,
                          "Node11": {"leaf111": 111},
                          },
                "Node2": {"leaf21": 21,
                          "Node21": {"leaf211": 211},
                          },
            })
        self.tree = tree

    def test_add_dictionary(self):
        self.tree.add_dictionary({
            "Node1": {"leaf12": 12,
                      "Node11": {"leaf111": 222,
                                 "Node111": {"leaf1111": 1111}, },
                      },
            "Node3": {
                "leaf31": 31},
        })
        nt.assert_equal(
            {"Node1": {"leaf11": 11,
                       "leaf12": 12,
                       "Node11": {"leaf111": 222,
                                  "Node111": {
                                      "leaf1111": 1111},
                                  },
                       },
             "Node2": {"leaf21": 21,
                       "Node21": {"leaf211": 211},
                       },
             "Node3": {"leaf31": 31},
             }, self.tree.as_dictionary())

    def test_add_signal_in_dictionary(self):
        tree = self.tree
        s = BaseSignal([1., 2, 3])
        s.axes_manager[0].name = 'x'
        s.axes_manager[0].units = 'ly'
        tree.add_dictionary({"_sig_signal name": s._to_dictionary()})
        nt.assert_is_instance(tree.signal_name, BaseSignal)
        np.testing.assert_array_equal(tree.signal_name.data, s.data)
        nt.assert_dict_equal(tree.signal_name.metadata.as_dictionary(),
                             s.metadata.as_dictionary())
        nt.assert_equal(tree.signal_name.axes_manager._get_axes_dicts(),
                        s.axes_manager._get_axes_dicts())

    def test_signal_to_dictionary(self):
        tree = self.tree
        s = BaseSignal([1., 2, 3])
        s.axes_manager[0].name = 'x'
        s.axes_manager[0].units = 'ly'
        tree.set_item('Some name', s)
        d = tree.as_dictionary()
        np.testing.assert_array_equal(d['_sig_Some name']['data'], s.data)
        d['_sig_Some name']['data'] = 0
        nt.assert_dict_equal(
            {
                "Node1": {
                    "leaf11": 11,
                    "Node11": {
                        "leaf111": 111},
                },
                "Node2": {
                    "leaf21": 21,
                    "Node21": {
                        "leaf211": 211},
                },
                "_sig_Some name": {
                    'axes': [
                        {
                            'name': 'x',
                            'navigate': False,
                                    'offset': 0.0,
                                    'scale': 1.0,
                                    'size': 3,
                                    'units': 'ly'}],
                    'data': 0,
                    'learning_results': {},
                    'metadata': {
                        'General': {
                            'title': ''},
                        'Signal': {
                            'binned': False,
                            'record_by': '',
                            'signal_origin': '',
                            'signal_type': ''},
                        '_HyperSpy': {
                            'Folding': {
                                'original_axes_manager': None,
                                'original_shape': None,
                                'unfolded': False,
                                'signal_unfolded': False}}},
                    'original_metadata': {},
                    'tmp_parameters': {}}},
            d)



import numpy as np
import nose.tools as nt

import hyperspy.api as hs


class TestCreateEELSModel:

    def setUp(self):
        s = hs.signals.EELSSpectrum(np.zeros(200))
        s.set_microscope_parameters(100, 10, 10)
        s.axes_manager[-1].offset = 150
        s.add_elements(("B", "C"))
        self.s = s

    def test_create_eelsmodel(self):
        from hyperspy.models.eelsmodel import EELSModel
        nt.assert_is_instance(self.s.create_model(), EELSModel)

    def test_auto_add_edges_true(self):
        m = self.s.create_model(auto_add_edges=True)
        cnames = [component.name for component in m]
        nt.assert_true("B_K" in cnames and "C_K" in cnames)

    def test_gos(self):
        m = self.s.create_model(auto_add_edges=True, GOS="hydrogenic")
        nt.assert_true(m["B_K"].GOS._name == "hydrogenic")

    def test_auto_add_background_true(self):
        m = self.s.create_model(auto_background=True)
        from hyperspy.components import PowerLaw
        is_pl_instance = [isinstance(c, PowerLaw) for c in m]
        nt.assert_true(True in is_pl_instance)

    def test_auto_add_edges_false(self):
        m = self.s.create_model(auto_background=False)
        from hyperspy.components import PowerLaw
        is_pl_instance = [isinstance(c, PowerLaw) for c in m]
        nt.assert_false(True in is_pl_instance)

    def test_auto_add_edges_false_names(self):
        m = self.s.create_model(auto_add_edges=False)
        cnames = [component.name for component in m]
        nt.assert_false("B_K" in cnames or "C_K" in cnames)

    def test_low_loss(self):
        ll = self.s.deepcopy()
        ll.axes_manager[-1].offset = -20
        m = self.s.create_model(ll=ll)
        nt.assert_is(m.low_loss, ll)


class TestEELSModel:

    def setUp(self):
        s = hs.signals.EELSSpectrum(np.zeros(200))
        s.set_microscope_parameters(100, 10, 10)
        s.axes_manager[-1].offset = 150
        s.add_elements(("B", "C"))
        self.m = s.create_model()

    def test_suspend_auto_fsw(self):
        m = self.m
        m["B_K"].fine_structure_width = 140.
        m.suspend_auto_fine_structure_width()
        m.enable_fine_structure()
        m.resolve_fine_structure()
        nt.assert_equal(140, m["B_K"].fine_structure_width)

    def test_resume_fsw(self):
        m = self.m
        m["B_K"].fine_structure_width = 140.
        m.suspend_auto_fine_structure_width()
        m.resume_auto_fine_structure_width()
        window = (m["C_K"].onset_energy.value -
                  m["B_K"].onset_energy.value -
                  hs.preferences.EELS.preedge_safe_window_width)
        m.enable_fine_structure()
        m.resolve_fine_structure()
        nt.assert_equal(window, m["B_K"].fine_structure_width)

    def test_get_first_ionization_edge_energy_C_B(self):
        nt.assert_equal(self.m._get_first_ionization_edge_energy(),
                        self.m["B_K"].onset_energy.value)

    def test_get_first_ionization_edge_energy_C(self):
        self.m["B_K"].active = False
        nt.assert_equal(self.m._get_first_ionization_edge_energy(),
                        self.m["C_K"].onset_energy.value)

    def test_get_first_ionization_edge_energy_None(self):
        self.m["B_K"].active = False
        self.m["C_K"].active = False
        nt.assert_is_none(self.m._get_first_ionization_edge_energy())

    def test_two_area_powerlaw_estimation_BC(self):
        self.m.signal.data = 2. * self.m.axis.axis ** (-3)  # A= 2, r=3
        self.m.signal.metadata.Signal.binned = False
        self.m.two_area_background_estimation()
        nt.assert_almost_equal(self.m._background_components[0].A.value,
                               2.1451237089380295)
        nt.assert_almost_equal(self.m._background_components[0].r.value,
                               3.0118980767392736)

    def test_two_area_powerlaw_estimation_C(self):
        self.m["B_K"].active = False
        self.m.signal.data = 2. * self.m.axis.axis ** (-3)  # A= 2, r=3
        self.m.signal.metadata.Signal.binned = False
        self.m.two_area_background_estimation()
        nt.assert_almost_equal(self.m._background_components[0].A.value,
                               2.3978438900878087)
        nt.assert_almost_equal(self.m._background_components[0].r.value,
                               3.031884021065014)

    def test_two_area_powerlaw_estimation_no_edge(self):
        self.m["B_K"].active = False
        self.m["C_K"].active = False
        self.m.signal.data = 2. * self.m.axis.axis ** (-3)  # A= 2, r=3
        self.m.signal.metadata.Signal.binned = False
        self.m.two_area_background_estimation()
        nt.assert_almost_equal(self.m._background_components[0].A.value,
                               2.6598803469440986)
        nt.assert_almost_equal(self.m._background_components[0].r.value,
                               3.0494030409062058)

    def test_get_start_energy_none(self):
        nt.assert_equal(self.m._get_start_energy(), 150)

    def test_get_start_energy_above(self):
        nt.assert_equal(self.m._get_start_energy(170), 170)

    def test_get_start_energy_below(self):
        nt.assert_equal(self.m._get_start_energy(100), 150)


class TestFitBackground:

    def setUp(self):
        s = hs.signals.EELSSpectrum(np.ones(200))
        s.set_microscope_parameters(100, 10, 10)
        s.axes_manager[-1].offset = 150
        CE = hs.material.elements.C.Atomic_properties.Binding_energies.K.onset_energy_eV
        BE = hs.material.elements.B.Atomic_properties.Binding_energies.K.onset_energy_eV
        s.isig[BE:] += 1
        s.isig[CE:] += 1
        s.add_elements(("Be", "B", "C"))
        self.m = s.create_model(auto_background=False)
        self.m.append(hs.model.components.Offset())

    def test_fit_background_B_C(self):
        self.m.fit_background()
        nt.assert_almost_equal(self.m["Offset"].offset.value, 1)
        nt.assert_true(self.m["B_K"].active)
        nt.assert_true(self.m["C_K"].active)

    def test_fit_background_C(self):
        self.m["B_K"].active = False
        self.m.fit_background()
        nt.assert_almost_equal(self.m["Offset"].offset.value, 1.71212121212)
        nt.assert_false(self.m["B_K"].active)
        nt.assert_true(self.m["C_K"].active)

    def test_fit_background_no_edge(self):
        self.m["B_K"].active = False
        self.m["C_K"].active = False
        self.m.fit_background()
        nt.assert_almost_equal(self.m["Offset"].offset.value, 2.13567839196)
        nt.assert_false(self.m["B_K"].active)
        nt.assert_false(self.m["C_K"].active)



# -*- coding: utf-8 -*-
# Copyright 2007-2016 The HyperSpy developers
#
# This file is part of  HyperSpy.
#
#  HyperSpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
#  HyperSpy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.

import math

import numpy as np

from hyperspy.component import Component

sqrt2pi = math.sqrt(2 * math.pi)
sigma2fwhm = 2 * math.sqrt(2 * math.log(2))


class Gaussian(Component):

    """Normalized gaussian function component

    .. math::

        f(x) = \\frac{a}{\sqrt{2\pi c^{2}}}e^{-\\frac{\left(x-b\\right)^{2}}{2c^{2}}}

    +------------+-----------+
    | Parameter  | Attribute |
    +------------+-----------+
    +------------+-----------+
    |     a      |     A     |
    +------------+-----------+
    |     b      |  centre   |
    +------------+-----------+
    |     c      |   sigma   |
    +------------+-----------+

    For convenience the `fwhm` attribute can be used to get and set
    the full-with-half-maximum.

    """

    def __init__(self, A=1., sigma=1., centre=0.):
        Component.__init__(self, ['A', 'sigma', 'centre'])
        self.A.value = A
        self.sigma.value = sigma
        self.centre.value = centre
        self._position = self.centre

        # Boundaries
        self.A.bmin = 0.
        self.A.bmax = None

        self.sigma.bmin = None
        self.sigma.bmax = None

        self.isbackground = False
        self.convolved = True

        # Gradients
        self.A.grad = self.grad_A
        self.sigma.grad = self.grad_sigma
        self.centre.grad = self.grad_centre

    def function(self, x):
        A = self.A.value
        sigma = self.sigma.value
        centre = self.centre.value
        return A * (1 / (sigma * sqrt2pi)) * np.exp(
            -(x - centre) ** 2 / (2 * sigma ** 2))

    def grad_A(self, x):
        return self.function(x) / self.A.value

    def grad_sigma(self, x):
        return ((x - self.centre.value) ** 2 * np.exp(-(x - self.centre.value) ** 2
                                                      / (2 * self.sigma.value ** 2)) * self.A.value) / (sqrt2pi *
                                                                                                        self.sigma.value ** 4) - (np.exp(-(x - self.centre.value) ** 2 / (2 *
                                                                                                                                                                          self.sigma.value ** 2)) * self.A.value) / (sqrt2pi * self.sigma.value ** 2)

    def grad_centre(self, x):
        return ((x - self.centre.value) * np.exp(-(x - self.centre.value) ** 2 /
                                                 (2 * self.sigma.value ** 2)) * self.A.value) / (sqrt2pi * self.sigma.value ** 3)

    def estimate_parameters(self, signal, x1, x2, only_current=False):
        """Estimate the gaussian by calculating the momenta.

        Parameters
        ----------
        signal : Signal instance
        x1 : float
            Defines the left limit of the spectral range to use for the
            estimation.
        x2 : float
            Defines the right limit of the spectral range to use for the
            estimation.

        only_current : bool
            If False estimates the parameters for the full dataset.

        Returns
        -------
        bool

        Notes
        -----
        Adapted from http://www.scipy.org/Cookbook/FittingData

        Examples
        --------

        >>> g = hs.model.components.Gaussian()
        >>> x = np.arange(-10,10, 0.01)
        >>> data = np.zeros((32,32,2000))
        >>> data[:] = g.function(x).reshape((1,1,2000))
        >>> s = hs.signals.Signal1D(data)
        >>> s.axes_manager._axes[-1].offset = -10
        >>> s.axes_manager._axes[-1].scale = 0.01
        >>> g.estimate_parameters(s, -10,10, False)

        """
        super(Gaussian, self)._estimate_parameters(signal)
        axis = signal.axes_manager.signal_axes[0]
        binned = signal.metadata.Signal.binned
        axis = signal.axes_manager.signal_axes[0]
        binned = signal.metadata.Signal.binned
        i1, i2 = axis.value_range_to_indices(x1, x2)
        X = axis.axis[i1:i2]
        if only_current is True:
            data = signal()[i1:i2]
            X_shape = (len(X),)
            i = 0
            center_shape = (1,)
        else:
            # TODO: write the rest of the code to estimate the parameters of
            # the full dataset
            i = axis.index_in_array
            data_gi = [slice(None), ] * len(signal.data.shape)
            data_gi[axis.index_in_array] = slice(i1, i2)
            data = signal.data[data_gi]
            X_shape = [1, ] * len(signal.data.shape)
            X_shape[axis.index_in_array] = data.shape[i]
            center_shape = list(data.shape)
            center_shape[i] = 1

        center = np.sum(X.reshape(X_shape) * data, i) / np.sum(data, i)

        sigma = np.sqrt(np.abs(np.sum((X.reshape(X_shape) - center.reshape(
            center_shape)) ** 2 * data, i) / np.sum(data, i)))
        height = data.max(i)
        if only_current is True:
            self.centre.value = center
            self.sigma.value = sigma
            self.A.value = height * sigma * sqrt2pi
            if binned is True:
                self.A.value /= axis.scale
            return True
        else:
            if self.A.map is None:
                self._create_arrays()
            self.A.map['values'][:] = height * sigma * sqrt2pi

            if binned is True:
                self.A.map['values'] /= axis.scale
            self.A.map['is_set'][:] = True
            self.sigma.map['values'][:] = sigma
            self.sigma.map['is_set'][:] = True
            self.centre.map['values'][:] = center
            self.centre.map['is_set'][:] = True
            self.fetch_stored_values()
            return True

    @property
    def fwhm(self):
        return self.sigma.value * sigma2fwhm

    @fwhm.setter
    def fwhm(self, value):
        self.sigma.value = value / sigma2fwhm



# -*- coding: utf-8 -*-
# Copyright 2007-2016 The HyperSpy developers
#
# This file is part of  HyperSpy.
#
#  HyperSpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
#  HyperSpy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.


import os

import numpy as np
from .generate_dm_testing_files import dm3_data_types

from nose.tools import assert_equal, assert_true
from hyperspy.io import load

my_path = os.path.dirname(__file__)

# When running the loading test the data of the files that passes it are
# stored in the following dict.
# TODO: fixtures should be used instead...
data_dict = {'dm3_1D_data': {},
             'dm3_2D_data': {},
             'dm3_3D_data': {}, }


def test_loading():
    dims = range(1, 4)
    for dim in dims:
        subfolder = 'dm3_%iD_data' % dim
        for key in dm3_data_types.keys():
            fname = "test-%s.dm3" % key
            filename = os.path.join(my_path, subfolder, fname)
            yield check_load, filename, subfolder, key


def test_dtypes():
    subfolder = 'dm3_1D_data'
    for key, data in data_dict[subfolder].items():
        yield check_dtype, data.dtype, np.dtype(dm3_data_types[key]), key

# TODO: the RGB data generated is not correct


# noinspection PyArgumentList
def test_content():
    for subfolder in data_dict:
        for key, data in data_dict[subfolder].items():
            if subfolder == 'dm3_1D_data':
                dat = np.arange(1, 3)
            elif subfolder == 'dm3_2D_data':
                dat = np.arange(1, 5).reshape(2, 2)
            elif subfolder == 'dm3_3D_data':
                dat = np.arange(1, 9).reshape(2, 2, 2)
            dat = dat.astype(dm3_data_types[key])
            if key in (8, 23):  # RGBA
                dat["A"][:] = 0
            yield check_content, data, dat, subfolder, key


def check_load(filename, subfolder, key):
    try:
        s = load(filename)
        ok = True
        # Store the data for the next tests
        data_dict[subfolder][key] = s.data
    except:
        ok = False
    assert_true(ok, msg='loading %s\\test-%i' % (subfolder, key))


def check_dtype(d1, d2, i):
    assert_equal(d1, d2, msg='test_dtype-%i' % i)


def check_content(dat1, dat2, subfolder, key):
    np.testing.assert_array_equal(dat1, dat2,
                                  err_msg='content %s type % i: '
                                  '\n%s not equal to \n%s' % (subfolder, key,
                                                              str(dat1),
                                                              str(dat2)))



""" Loads hyperspy as a regular python library, loads spectrums from files, does curve fitting, and plotting the model and original spectrum to a png file"""

import hyperspy.api as hs
import matplotlib.pyplot as plt

coreLossSpectrumFileName = "coreloss_spectrum.msa"
lowlossSpectrumFileName = "lowloss_spectrum.msa"

s = hs.load(coreLossSpectrumFileName).to_EELS()
s.add_elements(("Mn", "O"))
s.set_microscope_parameters(
    beam_energy=300,
    convergence_angle=24.6,
    collection_angle=13.6)

ll = hs.load(lowlossSpectrumFileName).to_EELS()

m = s.create_model(ll=ll)
m.enable_fine_structure()
m.multifit(kind="smart")
m.plot()

plt.savefig("model_original_spectrum_plot.png")



# Copyright 2007-2016 The HyperSpy developers
#
# This file is part of  HyperSpy.
#
#  HyperSpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
#  HyperSpy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.


import numpy as np
import nose.tools as nt

from hyperspy.signals import EDSTEMSpectrum, Simulation
from hyperspy.defaults_parser import preferences
from hyperspy.components import Gaussian
from hyperspy.misc.eds import utils as utils_eds


class Test_metadata:

    def setUp(self):
        # Create an empty spectrum
        s = EDSTEMSpectrum(np.ones((4, 2, 1024)))
        s.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time = 3.1
        s.metadata.Acquisition_instrument.TEM.beam_energy = 15.0
        self.signal = s

    def test_sum_live_time1(self):
        s = self.signal
        old_metadata = s.metadata.deepcopy()
        sSum = s.sum(0)
        nt.assert_equal(
            sSum.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time,
            3.1 * 2)
        # Check that metadata is unchanged
        print(old_metadata, s.metadata)      # Capture for comparison on error
        nt.assert_dict_equal(old_metadata.as_dictionary(),
                             s.metadata.as_dictionary(),
                             "Source metadata changed")

    def test_rebin_live_time(self):
        s = self.signal
        old_metadata = s.metadata.deepcopy()
        dim = s.axes_manager.shape
        s = s.rebin([dim[0] / 2, dim[1] / 2, dim[2]])
        nt.assert_equal(
            s.metadata.Acquisition_instrument.TEM.Detector.EDS.live_time,
            3.1 * 2 * 2)
        # Check that metadata is unchanged
        print(old_metadata, self.signal.metadata)    # Captured on error
        nt.assert_dict_equal(old_metadata.as_dictionary(),
                             self.signal.metadata.as_dictionary(),
                             "Source metadata changed")

    def test_add_elements(self):
        s = self.signal
        s.add_elements(['Al', 'Ni'])
        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])
        s.add_elements(['Al', 'Ni'])
        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])
        s.add_elements(["Fe", ])
        nt.assert_equal(s.metadata.Sample.elements, ['Al', "Fe", 'Ni'])
        s.set_elements(['Al', 'Ni'])
        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])

    def test_default_param(self):
        s = self.signal
        mp = s.metadata
        nt.assert_equal(
            mp.Acquisition_instrument.TEM.Detector.EDS.energy_resolution_MnKa,
            preferences.EDS.eds_mn_ka)

    def test_TEM_to_SEM(self):
        s = self.signal.inav[0, 0]
        signal_type = 'EDS_SEM'
        mp = s.metadata.Acquisition_instrument.TEM.Detector.EDS
        mp.energy_resolution_MnKa = 125.3
        sSEM = s.deepcopy()
        sSEM.set_signal_type(signal_type)
        mpSEM = sSEM.metadata.Acquisition_instrument.SEM.Detector.EDS
        results = [
            mp.energy_resolution_MnKa,
            signal_type]
        resultsSEM = [
            mpSEM.energy_resolution_MnKa,
            sSEM.metadata.Signal.signal_type]
        nt.assert_equal(results, resultsSEM)

    def test_get_calibration_from(self):
        s = self.signal
        scalib = EDSTEMSpectrum(np.ones(1024))
        energy_axis = scalib.axes_manager.signal_axes[0]
        energy_axis.scale = 0.01
        energy_axis.offset = -0.10
        s.get_calibration_from(scalib)
        nt.assert_equal(s.axes_manager.signal_axes[0].scale,
                        energy_axis.scale)


class Test_quantification:

    def setUp(self):
        s = EDSTEMSpectrum(np.ones([2, 1024]))
        energy_axis = s.axes_manager.signal_axes[0]
        energy_axis.scale = 1e-2
        energy_axis.units = 'keV'
        energy_axis.name = "Energy"
        s.set_microscope_parameters(beam_energy=200,
                                    live_time=3.1, tilt_stage=0.0,
                                    azimuth_angle=None, elevation_angle=35,
                                    energy_resolution_MnKa=130)
        elements = ['Al', 'Zn']
        xray_lines = ['Al_Ka', 'Zn_Ka']
        intensities = [300, 500]
        for i, xray_line in enumerate(xray_lines):
            gauss = Gaussian()
            line_energy, FWHM = s._get_line_energy(xray_line, FWHM_MnKa='auto')
            gauss.centre.value = line_energy
            gauss.A.value = intensities[i]
            gauss.sigma.value = FWHM
            s.data[:] += gauss.function(energy_axis.axis)

        s.set_elements(elements)
        s.add_lines(xray_lines)
        self.signal = s

    def test_quant_lorimer(self):
        s = self.signal
        kfactors = [1, 2.0009344042484134]
        intensities = s.get_lines_intensity()
        res = s.quantification(intensities, kfactors)
        np.testing.assert_allclose(res[0].data, np.array(
            [22.70779, 22.70779]), atol=1e-3)

    def test_quant_zeros(self):
        intens = np.array([[0.5, 0.5, 0.5],
                           [0., 0.5, 0.5],
                           [0.5, 0.0, 0.5],
                           [0.5, 0.5, 0.0],
                           [0.5, 0.0, 0.0]]).T
        quant = utils_eds.quantification_cliff_lorimer(
            intens, [1, 1, 3]).T
        np.testing.assert_allclose(
            quant,
            np.array([[0.2, 0.2, 0.6],
                      [0., 0.25, 0.75],
                      [0.25, 0., 0.75],
                      [0.5, 0.5, 0.],
                      [1., 0., 0.]]))


class Test_vacum_mask:

    def setUp(self):
        s = Simulation(np.array([np.linspace(0.001, 0.5, 20)] * 100).T)
        s.add_poissonian_noise()
        s = EDSTEMSpectrum(s.data)
        self.signal = s

    def test_vacuum_mask(self):
        s = self.signal
        nt.assert_true(s.vacuum_mask().data[0])
        nt.assert_false(s.vacuum_mask().data[-1])


class Test_get_lines_intentisity:

    def test_with_signals_examples(self):
        from hyperspy.misc.example_signals_loading import \
            load_1D_EDS_TEM_spectrum as EDS_TEM_Spectrum
        s = EDS_TEM_Spectrum()
        np.testing.assert_allclose(
            np.array([res.data[0] for res in s.get_lines_intensity()]),
            np.array([3710, 15872]))



# -*- coding: utf-8 -*-
# Copyright 2007-2015 The HyperSpy developers
#
# This file is part of  HyperSpy.
#
#  HyperSpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
#  HyperSpy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.

import os
import nose.tools as nt
import numpy as np

from hyperspy.io import load
from hyperspy.io_plugins.fei import load_ser_file

MY_PATH = os.path.dirname(__file__)


class TestFEIReader():

    def setUp(self):
        self.dirpathold = os.path.join(MY_PATH, 'FEI_old')
        self.dirpathnew = os.path.join(MY_PATH, 'FEI_new')

    def test_load_emi_old_new_format(self, verbose=True):
        # TIA old format
        fname0 = os.path.join(self.dirpathold, '64x64_TEM_images_acquire.emi')
        load(fname0, verbose=verbose)
        # TIA new format
        fname1 = os.path.join(self.dirpathnew, '128x128_TEM_acquire-sum1.emi')
        load(fname1, verbose=verbose)
        # TIA new format 1Go file
#        fname2= os.path.join('/mnt/data/test/64bits', 'CCD Preview-250-beam_damaged.emi')
#        load(fname2, verbose=verbose)
        # TIA new format 4Go file
#        fname2= os.path.join('/mnt/data/test/64bits', '11.22.31 CCD Preview-250-bean_damaged-bin2-0.2s.emi')
#        load(fname2, verbose=verbose)

    def test_load_image_content(self, verbose=True):
        # TEM image of the beam stop
        fname0 = os.path.join(self.dirpathold, '64x64_TEM_images_acquire.emi')
        s0 = load(fname0, verbose=verbose)
        data = np.load(fname0.replace('emi', 'npy'))
        np.testing.assert_array_equal(s0.data, data)

    def test_load_ser_reader_old_new_format(self, verbose=True):
        # test TIA old format
        fname0 = os.path.join(
            self.dirpathold, '64x64_TEM_images_acquire_1.ser')
        header0, data0 = load_ser_file(fname0, verbose=verbose)
        nt.assert_equal(header0['SeriesVersion'], 528)
        # test TIA new format
        fname1 = os.path.join(
            self.dirpathnew, '128x128_TEM_acquire-sum1_1.ser')
        header1, data1 = load_ser_file(fname1, verbose=verbose)
        nt.assert_equal(header1['SeriesVersion'], 544)

    def test_load_diffraction_point(self, verbose=True):
        fname0 = os.path.join(self.dirpathold, '64x64_diffraction_acquire.emi')
        s0 = load(fname0, verbose=verbose)
        nt.assert_equal(s0.data.shape, (64, 64))
        nt.assert_equal(s0.metadata.Signal.record_by, 'image')
        nt.assert_equal(
            s0.metadata.Acquisition_instrument.TEM.acquisition_mode, 'TEM')
        nt.assert_almost_equal(s0.axes_manager[0].scale, 0.10157, places=5)
        nt.assert_equal(s0.axes_manager[0].units, '1/nm')
        nt.assert_almost_equal(s0.axes_manager[1].scale, 0.10157, places=5)
        nt.assert_equal(s0.axes_manager[1].units, '1/nm')

    def test_load_diffraction_line_scan(self, verbose=True):
        fname0 = os.path.join(
            self.dirpathnew, '16x16-line_profile_horizontal_5x128x128_EDS.emi')
        s0 = load(fname0, verbose=verbose)
        # s0[0] contains EDS
        nt.assert_equal(s0[0].data.shape, (5, 4000))
        nt.assert_equal(s0[0].metadata.Signal.record_by, 'spectrum')
        nt.assert_equal(
            s0[0].metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
        nt.assert_almost_equal(s0[0].axes_manager[0].scale, 3.68864, places=5)
        nt.assert_equal(s0[0].axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s0[0].axes_manager[1].scale, 5.0, places=5)
        nt.assert_equal(s0[0].axes_manager[1].units, 'eV')
        # s0[1] contains diffraction patterns
        nt.assert_equal(s0[1].data.shape, (5, 128, 128))
        nt.assert_equal(s0[1].metadata.Signal.record_by, 'image')
        nt.assert_equal(
            s0[1].metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
        nt.assert_almost_equal(s0[1].axes_manager[0].scale, 3.68864, places=5)
        nt.assert_equal(s0[1].axes_manager[0].units, 'nm')
        nt.assert_equal(s0[1].axes_manager[1].units, '1/nm')
        nt.assert_almost_equal(s0[1].axes_manager[1].scale, 0.17435, places=5)
        nt.assert_almost_equal(s0[1].axes_manager[2].scale, 0.17435, places=5)
        nt.assert_equal(s0[1].axes_manager[2].units, '1/nm')

    def test_load_diffraction_area_scan(self, verbose=True):
        fname0 = os.path.join(
            self.dirpathnew, '16x16-diffraction_imagel_5x5x256x256_EDS.emi')
        s0 = load(fname0, verbose=verbose)
        # s0[0] contains EDS
        nt.assert_equal(s0[0].data.shape, (5, 5, 4000))
        nt.assert_equal(s0[0].metadata.Signal.record_by, 'spectrum')
        nt.assert_equal(
            s0[0].metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
        nt.assert_almost_equal(s0[0].axes_manager[0].scale, 1.87390, places=5)
        nt.assert_equal(s0[0].axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s0[0].axes_manager[1].scale, -1.87390, places=5)
        nt.assert_equal(s0[0].axes_manager[1].units, 'nm')
        nt.assert_almost_equal(s0[0].axes_manager[2].scale, 5.0, places=5)
        nt.assert_equal(s0[0].axes_manager[2].units, 'eV')
        # s0[1] contains diffraction patterns
        nt.assert_equal(s0[1].data.shape, (5, 5, 256, 256))
        nt.assert_equal(s0[1].metadata.Signal.record_by, 'image')
        nt.assert_equal(
            s0[1].metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
        nt.assert_almost_equal(s0[1].axes_manager[0].scale, -1.87390, places=5)
        nt.assert_equal(s0[1].axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s0[1].axes_manager[2].scale, 0.17435, places=5)
        nt.assert_equal(s0[1].axes_manager[2].units, '1/nm')

    def test_load_spectrum_point(self, verbose=True):
        fname0 = os.path.join(
            self.dirpathold, '16x16-point_spectrum-1x1024.emi')
        s0 = load(fname0, verbose=verbose)
        nt.assert_equal(s0.data.shape, (1, 1024))
        nt.assert_equal(s0.metadata.Signal.record_by, 'spectrum')
        nt.assert_equal(
            s0.metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
        # single spectrum should be imported as 1D data, not 2D
        nt.assert_almost_equal(
            s0.axes_manager[0].scale, 1000000000.0, places=5)
        nt.assert_equal(s0.axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s0.axes_manager[1].scale, 0.2, places=5)
        nt.assert_equal(s0.axes_manager[1].units, 'eV')

        fname1 = os.path.join(
            self.dirpathold, '16x16-2_point-spectra-2x1024.emi')
        s1 = load(fname1, verbose=verbose)
        nt.assert_equal(s1.data.shape, (2, 1024))
        nt.assert_equal(s1.metadata.Signal.record_by, 'spectrum')
        nt.assert_equal(
            s1.metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
        nt.assert_almost_equal(
            s0.axes_manager[0].scale, 1000000000.0, places=5)
        nt.assert_equal(s0.axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s0.axes_manager[1].scale, 0.2, places=5)
        nt.assert_equal(s0.axes_manager[1].units, 'eV')

    def test_load_spectrum_line_scan(self, verbose=True):
        fname0 = os.path.join(
            self.dirpathold, '16x16-line_profile_horizontal_10x1024.emi')
        s0 = load(fname0, verbose=verbose)
        nt.assert_equal(s0.data.shape, (10, 1024))
        nt.assert_equal(s0.metadata.Signal.record_by, 'spectrum')
        nt.assert_equal(
            s0.metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
        nt.assert_almost_equal(s0.axes_manager[0].scale, 0.12303, places=5)
        nt.assert_equal(s0.axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s0.axes_manager[1].scale, 0.2, places=5)
        nt.assert_equal(s0.axes_manager[1].units, 'eV')

        fname1 = os.path.join(
            self.dirpathold, '16x16-line_profile_diagonal_10x1024.emi')
        s1 = load(fname1, verbose=verbose)
        nt.assert_equal(s1.data.shape, (10, 1024))
        nt.assert_equal(s1.metadata.Signal.record_by, 'spectrum')
        nt.assert_equal(
            s1.metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
        nt.assert_almost_equal(s1.axes_manager[0].scale, 0.166318, places=5)
        nt.assert_equal(s1.axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s1.axes_manager[1].scale, 0.2, places=5)
        nt.assert_equal(s1.axes_manager[1].units, 'eV')

    def test_load_spectrum_area_scan(self, verbose=True):
        fname0 = os.path.join(
            self.dirpathold, '16x16-spectrum_image-5x5x1024.emi')
        s0 = load(fname0, verbose=verbose)
        nt.assert_equal(s0.data.shape, (5, 5, 1024))
        nt.assert_equal(s0.metadata.Signal.record_by, 'spectrum')
        nt.assert_equal(
            s0.metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
        nt.assert_almost_equal(s0.axes_manager[0].scale, 0.120539, places=5)
        nt.assert_equal(s0.axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s0.axes_manager[1].scale, -0.120539, places=5)
        nt.assert_equal(s0.axes_manager[1].units, 'nm')
        nt.assert_almost_equal(s0.axes_manager[2].scale, 0.2, places=5)
        nt.assert_equal(s0.axes_manager[2].units, 'eV')

    def test_load_spectrum_area_scan_not_square(self, verbose=True):
        fname0 = os.path.join(
            self.dirpathnew, '16x16-spectrum_image_5x5x4000-not_square.emi')
        s0 = load(fname0, verbose=verbose)
        nt.assert_equal(s0.data.shape, (5, 5, 4000))
        nt.assert_equal(s0.metadata.Signal.record_by, 'spectrum')
        nt.assert_equal(
            s0.metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
        nt.assert_almost_equal(s0.axes_manager[0].scale, 1.98591, places=5)
        nt.assert_equal(s0.axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s0.axes_manager[1].scale, -4.25819, places=5)
        nt.assert_equal(s0.axes_manager[1].units, 'nm')
        nt.assert_almost_equal(s0.axes_manager[2].scale, 5.0, places=5)
        nt.assert_equal(s0.axes_manager[2].units, 'eV')

    def test_load_search(self, verbose=True):
        fname0 = os.path.join(self.dirpathnew, '128x128-TEM_search.emi')
        s0 = load(fname0, verbose=verbose)
        nt.assert_equal(s0.data.shape, (128, 128))
        nt.assert_almost_equal(s0.axes_manager[0].scale, 5.26121, places=5)
        nt.assert_equal(s0.axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s0.axes_manager[1].scale, 5.26121, places=5)
        nt.assert_equal(s0.axes_manager[1].units, 'nm')

        fname1 = os.path.join(self.dirpathold, '16x16_STEM_BF_DF_search.emi')
        s1 = load(fname1, verbose=verbose)
        nt.assert_equal(len(s1), 2)
        for s in s1:
            nt.assert_equal(s.data.shape, (16, 16))
            nt.assert_equal(
                s.metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
            nt.assert_almost_equal(
                s.axes_manager[0].scale, 22.026285, places=5)
            nt.assert_equal(s.axes_manager[0].units, 'nm')
            nt.assert_almost_equal(
                s.axes_manager[1].scale, 22.026285, places=5)
            nt.assert_equal(s.axes_manager[1].units, 'nm')

    def test_load_stack_image_preview(self, verbose=True):
        fname0 = os.path.join(self.dirpathold, '64x64x5_TEM_preview.emi')
        s0 = load(fname0, verbose=verbose)
        nt.assert_equal(s0.data.shape, (5, 64, 64))
        nt.assert_equal(s0.metadata.Signal.record_by, 'image')
        nt.assert_equal(
            s0.metadata.Acquisition_instrument.TEM.acquisition_mode, 'TEM')
        nt.assert_almost_equal(s0.axes_manager[0].scale, 1.0, places=5)
        nt.assert_equal(s0.axes_manager[0].units, 'Unknown')
        nt.assert_almost_equal(s0.axes_manager[1].scale, 6.281833, places=5)
        nt.assert_equal(s0.axes_manager[1].units, 'nm')
        nt.assert_almost_equal(s0.axes_manager[2].scale, 6.281833, places=5)
        nt.assert_equal(s0.axes_manager[2].units, 'nm')

        fname2 = os.path.join(
            self.dirpathnew, '128x128x5-diffraction_preview.emi')
        s2 = load(fname2, verbose=verbose)
        nt.assert_equal(s2.data.shape, (5, 128, 128))
        nt.assert_almost_equal(s2.axes_manager[1].scale, 0.042464, places=5)
        nt.assert_equal(s0.axes_manager[0].units, 'Unknown')
        nt.assert_equal(s2.axes_manager[1].units, '1/nm')
        nt.assert_almost_equal(s2.axes_manager[2].scale, 0.042464, places=5)
        nt.assert_equal(s2.axes_manager[2].units, '1/nm')

        fname1 = os.path.join(
            self.dirpathold, '16x16x5_STEM_BF_DF_preview.emi')
        s1 = load(fname1, verbose=verbose)
        nt.assert_equal(len(s1), 2)
        for s in s1:
            nt.assert_equal(s.data.shape, (5, 16, 16))
            nt.assert_equal(
                s.metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
            nt.assert_almost_equal(s.axes_manager[0].scale, 1.0, places=5)
            nt.assert_equal(s.axes_manager[1].units, 'nm')
            nt.assert_almost_equal(
                s.axes_manager[1].scale, 21.510044, places=5)

    def test_load_acquire(self, verbose=True):
        fname0 = os.path.join(self.dirpathold, '64x64_TEM_images_acquire.emi')
        s0 = load(fname0, verbose=verbose)
        nt.assert_equal(s0.metadata.Signal.record_by, 'image')
        nt.assert_equal(
            s0.metadata.Acquisition_instrument.TEM.acquisition_mode, 'TEM')
        nt.assert_almost_equal(s0.axes_manager[0].scale, 6.281833, places=5)
        nt.assert_equal(s0.axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s0.axes_manager[1].scale, 6.281833, places=5)
        nt.assert_equal(s0.axes_manager[1].units, 'nm')

        fname1 = os.path.join(self.dirpathold, '16x16_STEM_BF_DF_acquire.emi')
        s1 = load(fname1, verbose=verbose)
        nt.assert_equal(len(s1), 2)
        for s in s1:
            nt.assert_equal(s.data.shape, (16, 16))
            nt.assert_equal(
                s.metadata.Acquisition_instrument.TEM.acquisition_mode, 'STEM')
            nt.assert_almost_equal(
                s.axes_manager[0].scale, 21.510044, places=5)
            nt.assert_equal(s.axes_manager[0].units, 'nm')
            nt.assert_almost_equal(
                s.axes_manager[1].scale, 21.510044, places=5)
            nt.assert_equal(s.axes_manager[1].units, 'nm')

    def test_read_STEM_TEM_mode(self, verbose=True):
        # TEM image
        fname0 = os.path.join(self.dirpathold, '64x64_TEM_images_acquire.emi')
        s0 = load(fname0, verbose=verbose)
        nt.assert_equal(
            s0.metadata.Acquisition_instrument.TEM.acquisition_mode, "TEM")
        # TEM diffraction
        fname1 = os.path.join(self.dirpathold, '64x64_diffraction_acquire.emi')
        s1 = load(fname1, verbose=verbose)
        nt.assert_equal(
            s1.metadata.Acquisition_instrument.TEM.acquisition_mode, "TEM")
        fname2 = os.path.join(self.dirpathold, '16x16_STEM_BF_DF_acquire.emi')
        # STEM diffraction
        s2 = load(fname2, verbose=verbose)
        nt.assert_equal(
            s2[0].metadata.Acquisition_instrument.TEM.acquisition_mode, "STEM")
        nt.assert_equal(
            s2[1].metadata.Acquisition_instrument.TEM.acquisition_mode, "STEM")

    def test_load_units_scale(self, verbose=True):
        # TEM image
        fname0 = os.path.join(self.dirpathold, '64x64_TEM_images_acquire.emi')
        s0 = load(fname0, verbose=verbose)
        nt.assert_almost_equal(s0.axes_manager[0].scale, 6.28183, places=5)
        nt.assert_equal(s0.axes_manager[0].units, 'nm')
        # TEM diffraction
        fname1 = os.path.join(self.dirpathold, '64x64_diffraction_acquire.emi')
        s1 = load(fname1, verbose=verbose)
        nt.assert_almost_equal(s1.axes_manager[0].scale, 0.10157, places=4)
        nt.assert_equal(s1.axes_manager[0].units, '1/nm')
        # STEM diffraction
        fname2 = os.path.join(self.dirpathold, '16x16_STEM_BF_DF_acquire.emi')
        s2 = load(fname2, verbose=verbose)
        nt.assert_equal(s2[0].axes_manager[0].units, 'nm')
        nt.assert_almost_equal(s2[0].axes_manager[0].scale, 21.5100, places=4)

    def test_guess_units_from_mode(self, verbose=True):
        from hyperspy.io_plugins.fei import guess_units_from_mode, \
            convert_xml_to_dict, get_xml_info_from_emi
        fname0_emi = os.path.join(
            self.dirpathold, '64x64_TEM_images_acquire.emi')
        fname0_ser = os.path.join(
            self.dirpathold, '64x64_TEM_images_acquire_1.ser')
        objects = get_xml_info_from_emi(fname0_emi)
        header0, data0 = load_ser_file(fname0_ser, verbose=verbose)
        objects_dict = convert_xml_to_dict(objects[0])

        unit = guess_units_from_mode(objects_dict, header0)
        nt.assert_equal(unit, 'meters')

        # objects is empty dictionary
        unit = guess_units_from_mode({}, header0)
        nt.assert_equal(unit, 'meters')

        # objects is empty dictionary
        unit = guess_units_from_mode({}, header0)
        nt.assert_equal(unit, 'meters')



import nose.tools
import numpy as np
from hyperspy.drawing.utils import contrast_stretching


class TestImageStretching:

    def setUp(self):
        self.data = np.arange(11).astype("float")
        self.data[-1] = np.nan

    def test_no_nans(self):
        data = self.data[:-1]
        bounds = contrast_stretching(data, 1)
        nose.tools.assert_tuple_equal(bounds, (
            np.percentile(data, 0.5),
            np.percentile(data, 99.5)))

    def test_nans(self):
        data = self.data[:-1]
        bounds = contrast_stretching(self.data, 1)
        nose.tools.assert_tuple_equal(bounds, (
            np.percentile(data, 0.5),
            np.percentile(data, 99.5)))

    @nose.tools.raises(ValueError)
    def test_out_of_range(self):
        contrast_stretching(self.data, -1)



# Copyright 2007-2016 The HyperSpy developers
#
# This file is part of  HyperSpy.
#
#  HyperSpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
#  HyperSpy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.


import numpy as np
from nose.tools import (
    assert_true,
    assert_equal,
    raises)

from hyperspy.signal import BaseSignal
from hyperspy import signals


class Test1D:

    def setUp(self):
        self.signal = BaseSignal(np.arange(10))
        self.data = self.signal.data.copy()

    def test_slice_None(self):
        s = self.signal.isig[:]
        d = self.data
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[0].offset,
                     self.signal.axes_manager._axes[0].offset)
        assert_equal(s.axes_manager._axes[0].scale,
                     self.signal.axes_manager._axes[0].scale)

    def test_reverse_slice(self):
        s = self.signal.isig[-1:1:-1]
        d = self.data[-1:1:-1]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[0].offset, 9)
        assert_equal(s.axes_manager._axes[0].scale,
                     self.signal.axes_manager._axes[0].scale * -1)

    def test_slice_out_of_axis(self):
        np.testing.assert_array_equal(
            self.signal.isig[-1.:].data, self.signal.data)
        np.testing.assert_array_equal(
            self.signal.isig[
                :11.].data, self.signal.data)

    @raises(ValueError)
    def test_step0_slice(self):
        self.signal.isig[::0]

    def test_index(self):
        s = self.signal.isig[3]
        assert_equal(s.data, 3)
        assert_equal(len(s.axes_manager._axes), 1)
        assert_equal(s.data.shape, (1,))

    def test_float_index(self):
        s = self.signal.isig[3.4]
        assert_equal(s.data, 3)
        assert_equal(len(s.axes_manager._axes), 1)
        assert_equal(s.data.shape, (1,))

    def test_signal_indexer_slice(self):
        s = self.signal.isig[1:-1]
        d = self.data[1:-1]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[0].offset, 1)
        assert_equal(s.axes_manager._axes[0].scale,
                     self.signal.axes_manager._axes[0].scale)

    def test_signal_indexer_reverse_slice(self):
        s = self.signal.isig[-1:1:-1]
        d = self.data[-1:1:-1]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[0].offset, 9)
        assert_equal(s.axes_manager._axes[0].scale,
                     self.signal.axes_manager._axes[0].scale * -1)

    def test_signal_indexer_step2_slice(self):
        s = self.signal.isig[1:-1:2]
        d = self.data[1:-1:2]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[0].offset, 1)
        assert_equal(np.sign(s.axes_manager._axes[0].scale),
                     np.sign(self.signal.axes_manager._axes[0].scale))
        assert_equal(s.axes_manager._axes[0].scale,
                     self.signal.axes_manager._axes[0].scale * 2.)

    def test_signal_indexer_index(self):
        s = self.signal.isig[3]
        assert_equal(s.data, 3)
        assert_equal(len(s.axes_manager._axes), 1)
        assert_equal(s.data.shape, (1,))

    @raises(IndexError)
    def test_navigation_indexer_navdim0(self):
        self.signal.inav[3]

    def test_minus_one_index(self):
        s = self.signal.isig[-1]
        assert_equal(s.data, self.data[-1])


class Test2D:

    def setUp(self):
        self.signal = BaseSignal(np.arange(24).reshape(6, 4))
        self.signal.axes_manager.set_signal_dimension(2)
        self.data = self.signal.data.copy()

    def test_index(self):
        s = self.signal.isig[3, 2]
        assert_equal(s.data[0], 11)
        assert_equal(len(s.axes_manager._axes), 1)
        assert_equal(s.data.shape, (1,))

    def test_partial(self):
        s = self.signal.isig[3, 2:5]
        np.testing.assert_array_equal(s.data, [11, 15, 19])
        assert_equal(len(s.axes_manager._axes), 1)
        assert_equal(s.data.shape, (3,))


class Test3D_SignalDim0:

    def setUp(self):
        self.signal = BaseSignal(np.arange(24).reshape((2, 3, 4)))
        self.data = self.signal.data.copy()
        self.signal.axes_manager._axes[2].navigate = True

    @raises(IndexError)
    def test_signal_indexer_signal_dim0_idx_error1(self):
        s = self.signal
        s.isig[:].data

    @raises(IndexError)
    def test_signal_indexer_signal_dim0_idx_error2(self):
        s = self.signal
        s.isig[:, :].data

    @raises(IndexError)
    def test_signal_indexer_signal_dim0_idx_error3(self):
        s = self.signal
        s.isig[0]

    def test_navigation_indexer_signal_dim0(self):
        s = self.signal
        np.testing.assert_array_equal(s.data, s.inav[:].data)


class Test3D_Navigate_0_and_1:

    def setUp(self):
        self.signal = BaseSignal(np.arange(24).reshape((2, 3, 4)))
        self.data = self.signal.data.copy()
        self.signal.axes_manager._axes[0].navigate = True
        self.signal.axes_manager._axes[1].navigate = True
        self.signal.axes_manager._axes[2].navigate = False

    def test_1px_navigation_indexer_slice(self):
        s = self.signal.inav[1:2]
        d = self.data[:, 1:2]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[1].offset, 1)
        assert_equal(s.axes_manager._axes[1].size, 1)
        assert_equal(s.axes_manager._axes[1].scale,
                     self.signal.axes_manager._axes[1].scale)

    def test_1px_signal_indexer_slice(self):
        s = self.signal.isig[1:2]
        d = self.data[:, :, 1:2]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager.signal_axes[0].offset, 1)
        assert_equal(s.axes_manager.signal_axes[0].size, 1)
        assert_equal(s.axes_manager.signal_axes[0].scale,
                     self.signal.axes_manager.signal_axes[0].scale)

    def test_signal_indexer_slice_variance_signal(self):
        s1 = self.signal
        s1.estimate_poissonian_noise_variance()
        s1_1 = s1.isig[1:2]
        np.testing.assert_array_equal(
            s1.metadata.Signal.Noise_properties.variance.data[:, :, 1:2],
            s1_1.metadata.Signal.Noise_properties.variance.data)

    def test_navigation_indexer_slice_variance_signal(self):
        s1 = self.signal
        s1.estimate_poissonian_noise_variance()
        s1_1 = s1.inav[1:2]
        np.testing.assert_array_equal(
            s1.metadata.Signal.Noise_properties.variance.data[:, 1:2],
            s1_1.metadata.Signal.Noise_properties.variance.data)

    def test_signal_indexer_slice_variance_float(self):
        s1 = self.signal
        s1.metadata.set_item("Signal.Noise_properties.variance", 1.2)
        s1_1 = s1.isig[1:2]
        assert_equal(
            s1.metadata.Signal.Noise_properties.variance,
            s1_1.metadata.Signal.Noise_properties.variance)

    def test_navigation_indexer_slice_variance_float(self):
        s1 = self.signal
        s1.metadata.set_item("Signal.Noise_properties.variance", 1.2)
        s1_1 = s1.inav[1:2]
        assert_equal(
            s1.metadata.Signal.Noise_properties.variance,
            s1_1.metadata.Signal.Noise_properties.variance)

    def test_dimension_when_indexing(self):
        s = self.signal.inav[0]
        assert_equal(s.data.shape, self.data[:, 0, :].shape)

    def test_dimension_when_slicing(self):
        s = self.signal.inav[0:1]
        assert_equal(s.data.shape, self.data[:, 0:1, :].shape)


class Test3D_Navigate_1:

    def setUp(self):
        self.signal = BaseSignal(np.arange(24).reshape((2, 3, 4)))
        self.data = self.signal.data.copy()
        self.signal.axes_manager._axes[0].navigate = False
        self.signal.axes_manager._axes[1].navigate = True
        self.signal.axes_manager._axes[2].navigate = False

    def test_1px_navigation_indexer_slice(self):
        s = self.signal.inav[1:2]
        d = self.data[:, 1:2]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[1].offset, 1)
        assert_equal(s.axes_manager._axes[1].size, 1)
        assert_equal(s.axes_manager._axes[1].scale,
                     self.signal.axes_manager._axes[1].scale)

    def test_1px_signal_indexer_slice(self):
        s = self.signal.isig[1:2]
        d = self.data[:, :, 1:2]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager.signal_axes[0].offset, 1)
        assert_equal(s.axes_manager.signal_axes[0].size, 1)
        assert_equal(s.axes_manager.signal_axes[0].scale,
                     self.signal.axes_manager.signal_axes[0].scale)

    def test_subclass_assignment(self):
        im = self.signal.as_signal2D((-2, -1))
        assert_true(isinstance(im.isig[0], signals.Signal1D))


class TestFloatArguments:

    def setUp(self):
        self.signal = BaseSignal(np.arange(10))
        self.signal.axes_manager[0].scale = 0.5
        self.signal.axes_manager[0].offset = 0.25
        self.data = self.signal.data.copy()

    def test_float_start(self):
        s = self.signal.isig[0.75:-1]
        d = self.data[1:-1]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[0].offset, 0.75)
        assert_equal(s.axes_manager._axes[0].scale,
                     self.signal.axes_manager._axes[0].scale)

    def test_float_end(self):
        s = self.signal.isig[1:4.75]
        d = self.data[1:-1]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[0].offset, 0.75)
        assert_equal(s.axes_manager._axes[0].scale,
                     self.signal.axes_manager._axes[0].scale)

    def test_float_both(self):
        s = self.signal.isig[0.75:4.75]
        d = self.data[1:-1]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[0].offset, 0.75)
        assert_equal(s.axes_manager._axes[0].scale,
                     self.signal.axes_manager._axes[0].scale)

    def test_float_step(self):
        s = self.signal.isig[::1.1]
        d = self.data[::2]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[0].offset, 0.25)
        assert_equal(s.axes_manager._axes[0].scale,
                     self.signal.axes_manager._axes[0].scale * 2)

    def test_negative_float_step(self):
        s = self.signal.isig[::-1.1]
        d = self.data[::-2]
        np.testing.assert_array_equal(s.data, d)
        assert_equal(s.axes_manager._axes[0].offset, 4.75)
        assert_equal(s.axes_manager._axes[0].scale,
                     self.signal.axes_manager._axes[0].scale * -2)


class TestEllipsis:

    def setUp(self):
        self.signal = BaseSignal(np.arange(2 ** 5).reshape(
            (2, 2, 2, 2, 2)))
        self.data = self.signal.data.copy()

    def test_in_between(self):
        s = self.signal.inav[0, ..., 0]
        np.testing.assert_array_equal(s.data, self.data[0, ..., 0, :])

    def test_ellipsis_navigation(self):
        s = self.signal.inav[..., 0]
        np.testing.assert_array_equal(s.data, self.data[0, ...])

    def test_ellipsis_navigation2(self):
        self.signal.axes_manager._axes[-2].navigate = False
        self.signal.axes_manager._axes[-3].navigate = False
        s = self.signal.isig[..., 0]
        np.testing.assert_array_equal(s.data, self.data[:, :, 0, ...])



# Copyright 2007-2016 The HyperSpy developers
#
# This file is part of  HyperSpy.
#
#  HyperSpy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
#  HyperSpy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with  HyperSpy.  If not, see <http://www.gnu.org/licenses/>.


import numpy as np
import nose.tools as nt

from hyperspy.signals import EDSSEMSpectrum
from hyperspy.defaults_parser import preferences
from hyperspy.components import Gaussian
from hyperspy import utils


class Test_metadata:

    def setUp(self):
        # Create an empty spectrum
        s = EDSSEMSpectrum(np.ones((4, 2, 1024)))
        s.axes_manager.signal_axes[0].scale = 1e-3
        s.axes_manager.signal_axes[0].units = "keV"
        s.axes_manager.signal_axes[0].name = "Energy"
        s.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time = 3.1
        s.metadata.Acquisition_instrument.SEM.beam_energy = 15.0
        s.metadata.Acquisition_instrument.SEM.tilt_stage = -38
        s.metadata.Acquisition_instrument.SEM.Detector.EDS.azimuth_angle = 63
        s.metadata.Acquisition_instrument.SEM.Detector.EDS.elevation_angle = 35
        self.signal = s

    def test_sum_live_time(self):
        s = self.signal
        old_metadata = s.metadata.deepcopy()
        sSum = s.sum(0)
        nt.assert_equal(
            sSum.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time,
            3.1 * 2)
        # Check that metadata is unchanged
        print(old_metadata, s.metadata)      # Capture for comparison on error
        nt.assert_dict_equal(old_metadata.as_dictionary(),
                             s.metadata.as_dictionary(),
                             "Source metadata changed")

    def test_rebin_live_time(self):
        s = self.signal
        old_metadata = s.metadata.deepcopy()
        dim = s.axes_manager.shape
        s = s.rebin([dim[0] / 2, dim[1] / 2, dim[2]])
        nt.assert_equal(
            s.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time,
            3.1 *
            2 *
            2)
        # Check that metadata is unchanged
        print(old_metadata, self.signal.metadata)    # Captured on error
        nt.assert_dict_equal(old_metadata.as_dictionary(),
                             self.signal.metadata.as_dictionary(),
                             "Source metadata changed")

    def test_add_elements(self):
        s = self.signal
        s.add_elements(['Al', 'Ni'])
        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])
        s.add_elements(['Al', 'Ni'])
        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])
        s.add_elements(["Fe", ])
        nt.assert_equal(s.metadata.Sample.elements, ['Al', "Fe", 'Ni'])
        s.set_elements(['Al', 'Ni'])
        nt.assert_equal(s.metadata.Sample.elements, ['Al', 'Ni'])

    def test_add_lines(self):
        s = self.signal
        s.add_lines(lines=())
        nt.assert_equal(s.metadata.Sample.xray_lines, [])
        s.add_lines(("Fe_Ln",))
        nt.assert_equal(s.metadata.Sample.xray_lines, ["Fe_Ln"])
        s.add_lines(("Fe_Ln",))
        nt.assert_equal(s.metadata.Sample.xray_lines, ["Fe_Ln"])
        s.add_elements(["Ti", ])
        s.add_lines(())
        nt.assert_equal(
            s.metadata.Sample.xray_lines, ['Fe_Ln', 'Ti_La'])
        s.set_lines((), only_one=False, only_lines=False)
        nt.assert_equal(s.metadata.Sample.xray_lines,
                        ['Fe_La', 'Fe_Lb3', 'Fe_Ll', 'Fe_Ln', 'Ti_La',
                         'Ti_Lb3', 'Ti_Ll', 'Ti_Ln'])
        s.metadata.Acquisition_instrument.SEM.beam_energy = 0.4
        s.set_lines((), only_one=False, only_lines=False)
        nt.assert_equal(s.metadata.Sample.xray_lines, ['Ti_Ll'])

    def test_add_lines_auto(self):
        s = self.signal
        s.axes_manager.signal_axes[0].scale = 1e-2
        s.set_elements(["Ti", "Al"])
        s.set_lines(['Al_Ka'])
        nt.assert_equal(
            s.metadata.Sample.xray_lines, ['Al_Ka', 'Ti_Ka'])

        del s.metadata.Sample.xray_lines
        s.set_elements(['Al', 'Ni'])
        s.add_lines()
        nt.assert_equal(
            s.metadata.Sample.xray_lines, ['Al_Ka', 'Ni_Ka'])
        s.metadata.Acquisition_instrument.SEM.beam_energy = 10.0
        s.set_lines([])
        nt.assert_equal(
            s.metadata.Sample.xray_lines, ['Al_Ka', 'Ni_La'])
        s.metadata.Acquisition_instrument.SEM.beam_energy = 200
        s.set_elements(['Au', 'Ni'])
        s.set_lines([])
        nt.assert_equal(s.metadata.Sample.xray_lines,
                        ['Au_La', 'Ni_Ka'])

    def test_default_param(self):
        s = self.signal
        mp = s.metadata
        nt.assert_equal(
            mp.Acquisition_instrument.SEM.Detector.EDS.energy_resolution_MnKa,
            preferences.EDS.eds_mn_ka)

    def test_SEM_to_TEM(self):
        s = self.signal.inav[0, 0]
        signal_type = 'EDS_TEM'
        mp = s.metadata
        mp.Acquisition_instrument.SEM.Detector.EDS.energy_resolution_MnKa = \
            125.3
        sTEM = s.deepcopy()
        sTEM.set_signal_type(signal_type)
        mpTEM = sTEM.metadata
        results = [
            mp.Acquisition_instrument.SEM.Detector.EDS.energy_resolution_MnKa,
            signal_type]
        resultsTEM = [
            (mpTEM.Acquisition_instrument.TEM.Detector.EDS.
             energy_resolution_MnKa),
            mpTEM.Signal.signal_type]
        nt.assert_equal(results, resultsTEM)

    def test_get_calibration_from(self):
        s = self.signal
        scalib = EDSSEMSpectrum(np.ones(1024))
        energy_axis = scalib.axes_manager.signal_axes[0]
        energy_axis.scale = 0.01
        energy_axis.offset = -0.10
        s.get_calibration_from(scalib)
        nt.assert_equal(s.axes_manager.signal_axes[0].scale, energy_axis.scale)

    def test_take_off_angle(self):
        s = self.signal
        nt.assert_equal(s.get_take_off_angle(), 12.886929785732487)


class Test_get_lines_intentisity:

    def setUp(self):
        # Create an empty spectrum
        s = EDSSEMSpectrum(np.zeros((2, 2, 3, 100)))
        energy_axis = s.axes_manager.signal_axes[0]
        energy_axis.scale = 0.04
        energy_axis.units = 'keV'
        energy_axis.name = "Energy"
        g = Gaussian()
        g.sigma.value = 0.05
        g.centre.value = 1.487
        s.data[:] = g.function(energy_axis.axis)
        s.metadata.Acquisition_instrument.SEM.Detector.EDS.live_time = 3.1
        s.metadata.Acquisition_instrument.SEM.beam_energy = 15.0
        self.signal = s

    def test(self):
        s = self.signal
        sAl = s.get_lines_intensity(["Al_Ka"],
                                    plot_result=False,
                                    integration_windows=5)[0]
        np.testing.assert_allclose(24.99516, sAl.data[0, 0, 0], atol=1e-3)
        sAl = s.inav[0].get_lines_intensity(
            ["Al_Ka"], plot_result=False, integration_windows=5)[0]
        np.testing.assert_allclose(24.99516, sAl.data[0, 0], atol=1e-3)
        sAl = s.inav[0, 0].get_lines_intensity(
            ["Al_Ka"], plot_result=False, integration_windows=5)[0]
        np.testing.assert_allclose(24.99516, sAl.data[0], atol=1e-3)
        sAl = s.inav[0, 0, 0].get_lines_intensity(
            ["Al_Ka"], plot_result=False, integration_windows=5)[0]
        np.testing.assert_allclose(24.99516, sAl.data, atol=1e-3)
        s.axes_manager[-1].offset = 1.0
        sC = s.get_lines_intensity(["C_Ka"], plot_result=False)
        nt.assert_equal(len(sC), 0)
        nt.assert_true(sAl.metadata.Sample.elements, ["Al"])
        nt.assert_true(sAl.metadata.Sample.xray_lines, ["Al_Ka"])

    def test_eV(self):
        s = self.signal
        energy_axis = s.axes_manager.signal_axes[0]
        energy_axis.scale = 40
        energy_axis.units = 'eV'

        sAl = s.get_lines_intensity(["Al_Ka"],
                                    plot_result=False,
                                    integration_windows=5)[0]
        np.testing.assert_allclose(24.99516, sAl.data[0, 0, 0], atol=1e-3)

    def test_background_substraction(self):
        s = self.signal
        intens = s.get_lines_intensity(["Al_Ka"], plot_result=False)[0].data
        s += 1.
        np.testing.assert_allclose(s.estimate_background_windows(
            xray_lines=["Al_Ka"])[0, 0], 1.25666201, atol=1e-3)
        np.testing.assert_allclose(
            s.get_lines_intensity(
                ["Al_Ka"],
                background_windows=s.estimate_background_windows(
                    [4, 4], xray_lines=["Al_Ka"]),
                plot_result=False)[0].data,
            intens, atol=1e-3)

    def test_estimate_integration_windows(self):
        s = self.signal
        np.testing.assert_allclose(
            s.estimate_integration_windows(3.0, ["Al_Ka"]),
            [[1.371, 1.601]], atol=1e-2)

    def test_with_signals_examples(self):
        from hyperspy.misc.example_signals_loading import \
            load_1D_EDS_SEM_spectrum as EDS_SEM_Spectrum
        s = EDS_SEM_Spectrum()
        np.testing.assert_allclose(
            utils.stack(s.get_lines_intensity()).data.squeeze(),
            np.array([84163, 89063, 96117, 96700, 99075]))


class Test_tools_bulk:

    def setUp(self):
        s = EDSSEMSpectrum(np.ones(1024))
        s.metadata.Acquisition_instrument.SEM.beam_energy = 5.0
        energy_axis = s.axes_manager.signal_axes[0]
        energy_axis.scale = 0.01
        energy_axis.units = 'keV'
        s.set_elements(['Al', 'Zn'])
        s.add_lines()
        self.signal = s

    def test_electron_range(self):
        s = self.signal
        mp = s.metadata
        elec_range = utils.eds.electron_range(
            mp.Sample.elements[0],
            mp.Acquisition_instrument.SEM.beam_energy,
            density='auto',
            tilt=mp.Acquisition_instrument.SEM.tilt_stage)
        np.testing.assert_allclose(elec_range, 0.41350651162374225)

    def test_xray_range(self):
        s = self.signal
        mp = s.metadata
        xr_range = utils.eds.xray_range(
            mp.Sample.xray_lines[0],
            mp.Acquisition_instrument.SEM.beam_energy,
            density=4.37499648818)
        np.testing.assert_allclose(xr_range, 0.1900368800933955)


class Test_energy_units:

    def setUp(self):
        s = EDSSEMSpectrum(np.ones(1024))
        s.metadata.Acquisition_instrument.SEM.beam_energy = 5.0
        s.axes_manager.signal_axes[0].units = 'keV'
        s.set_microscope_parameters(energy_resolution_MnKa=130)
        self.signal = s

    def test_beam_energy(self):
        s = self.signal
        nt.assert_equal(s._get_beam_energy(), 5.0)
        s.axes_manager.signal_axes[0].units = 'eV'
        nt.assert_equal(s._get_beam_energy(), 5000.0)
        s.axes_manager.signal_axes[0].units = 'keV'

    def test_line_energy(self):
        s = self.signal
        nt.assert_equal(s._get_line_energy('Al_Ka'), 1.4865)
        s.axes_manager.signal_axes[0].units = 'eV'
        nt.assert_equal(s._get_line_energy('Al_Ka'), 1486.5)
        s.axes_manager.signal_axes[0].units = 'keV'

        np.testing.assert_allclose(s._get_line_energy('Al_Ka', FWHM_MnKa='auto'),
                                   (1.4865, 0.07661266213883969))
        np.testing.assert_allclose(s._get_line_energy('Al_Ka', FWHM_MnKa=128),
                                   (1.4865, 0.073167615787314))



# -*- coding: utf-8 -*-
from nose.tools import assert_equal

import hyperspy.signals
import hyperspy.signal


def test_signal_record_by():
    assert_equal(hyperspy.signal.BaseSignal._record_by, "")


def test_signal_signal_type():
    assert_equal(hyperspy.signal.BaseSignal._signal_type, "")


def test_signal_signal_origin():
    assert_equal(hyperspy.signal.BaseSignal._signal_origin, "")


def test_spectrum_record_by():
    assert_equal(hyperspy.signals.Spectrum._record_by, "spectrum")


def test_spectrum_signal_type():
    assert_equal(hyperspy.signals.Spectrum._signal_type, "")


def test_spectrum_signal_origin():
    assert_equal(hyperspy.signals.Spectrum._signal_origin, "")


def test_image_record_by():
    assert_equal(hyperspy.signals.Image._record_by, "image")


def test_image_signal_type():
    assert_equal(hyperspy.signals.Image._signal_type, "")


def test_image_signal_origin():
    assert_equal(hyperspy.signals.Image._signal_origin, "")


def test_simulation_record_by():
    assert_equal(hyperspy.signals.Simulation._record_by, "")


def test_simulation_signal_type():
    assert_equal(hyperspy.signals.Simulation._signal_type, "")


def test_simulation_signal_origin():
    assert_equal(hyperspy.signals.Simulation._signal_origin, "simulation")


def test_spectrum_simulation_record_by():
    assert_equal(hyperspy.signals.SpectrumSimulation._record_by, "spectrum")


def test_spectrum_simulation_signal_type():
    assert_equal(hyperspy.signals.SpectrumSimulation._signal_type, "")


def test_spectrum_simulation_signal_origin():
    assert_equal(hyperspy.signals.SpectrumSimulation._signal_origin,
                 "simulation")


def test_image_simulation_record_by():
    assert_equal(hyperspy.signals.ImageSimulation._record_by, "image")


def test_image_simulation_signal_type():
    assert_equal(hyperspy.signals.ImageSimulation._signal_type, "")


def test_image_simulation_signal_origin():
    assert_equal(hyperspy.signals.ImageSimulation._signal_origin,
                 "simulation")


def test_eels_record_by():
    assert_equal(hyperspy.signals.EELSSpectrum._record_by, "spectrum")


def test_eels_signal_type():
    assert_equal(hyperspy.signals.EELSSpectrum._signal_type, "EELS")


def test_eels_signal_origin():
    assert_equal(hyperspy.signals.EELSSpectrum._signal_origin, "")
